package com.gic.enterprise.context;

import com.gic.commons.util.ToolUtil;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.utils.UserDetail;
import com.gic.redis.data.util.RedisUtil;
import org.apache.commons.lang.StringUtils;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.concurrent.TimeUnit;

public class UserContext {
    public static final Long expireLength = 1l;
    private static UserContext userContext = new UserContext();

    public static UserContext getContext(){
        return userContext;
    }

    public UserDetail get(){
        String token = getToken();
        if(StringUtils.isBlank(token)){
            return new UserDetail();
        }
        String key = Constants.TOKEN_KEY + token;
        Object cache = RedisUtil.getCache(key);
        if(cache != null){
            return (UserDetail) cache;
        }
        return new UserDetail();
    }

    public void init(UserDetail userDetail){
        String uuId = ToolUtil.randomUUID();
        String key = Constants.TOKEN_KEY+uuId;
        RedisUtil.setCache(key, userDetail, expireLength, TimeUnit.HOURS);
        Cookie cookie = new Cookie(Constants.USER_TOKEN, uuId);
        cookie.setPath("/");
        HttpServletResponse response = RequestContext.getContext().getResponse();
        if(response != null){
            response.addCookie(cookie);
        }
    }

    public void refresh(){
        String key = Constants.TOKEN_KEY + getToken();
        RedisUtil.getRedisClient().getBucket(key).expire(expireLength, TimeUnit.HOURS);
    }

    public void destory(){
        String key = Constants.TOKEN_KEY + getToken();
        RedisUtil.delCache(key);
    }

    public String getToken(){
        String token = null;
        HttpServletRequest request = RequestContext.getContext().getRequest();
        if (request != null){
            Cookie[] cookies = request.getCookies();
            if(cookies != null){
                for (Cookie cookie : cookies) {
                    if (Constants.USER_TOKEN.equals(cookie.getName())) {
                        token = cookie.getValue();
                        break;
                    }
                }
            }
        }
        return token;
    }
}
