package com.gic.enterprise.interceptor;

import com.alibaba.fastjson.JSON;
import com.gic.enterprise.ano.IgnoreLogin;
import com.gic.enterprise.base.UserInfo;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.context.UserContext;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.exception.CommonException;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.dubbo.rpc.RpcContext;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;


public class AuthInterceptor extends HandlerInterceptorAdapter {

    private static final Logger log = LogManager.getLogger(AuthInterceptor.class);

    private static final Map<String, Object> ignoreUriBeforeLoginMap = new HashMap<>(16);
    private static final Map<String, Object> ignoreUriAfterLoginMap = new HashMap<>(16);
    static {
        ignoreUriBeforeLoginMap.put("/gic-auth-web/login", "1");
        ignoreUriBeforeLoginMap.put("/gic-auth-web/logout", "1");
        ignoreUriBeforeLoginMap.put("/gic-auth-web/list-enterprise-by-phone", "1");
        ignoreUriAfterLoginMap.put("/gic-auth-web/login-user-menu", "1");
        ignoreUriAfterLoginMap.put("/gic-auth-web/login-user-operation-item", "1");
        ignoreUriBeforeLoginMap.put("/api-plug/dict-district-list", "1");
        ignoreUriBeforeLoginMap.put("/api-plug/upload-images", "1");
        ignoreUriBeforeLoginMap.put("/api-plug/query-provinces", "1");
        ignoreUriBeforeLoginMap.put("/api-plug/query-city", "1");
        ignoreUriBeforeLoginMap.put("/api-plug/query-county", "1");
        ignoreUriBeforeLoginMap.put("/api-plug/query-district", "1");
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object object) throws Exception {
        String token = UserContext.getContext().getToken();
        RpcContext.getContext().getAttachments().put(Constants.USER_TOKEN, token);
        System.out.println(JSON.toJSONString(RpcContext.getContext().getArguments()));


        if (!(object.getClass().isAssignableFrom(HandlerMethod.class))) {
            return true;
        }
        HandlerMethod handler = (HandlerMethod) object;
        IgnoreLogin ignoreLogin = handler.getMethodAnnotation(IgnoreLogin.class);
        if (null != ignoreLogin) {
            return true;
        }

        //获取uri
        String uri = request.getRequestURI();
        String isControl = request.getHeader("isControl");
        log.info("用户接口访问标志：{}", isControl);
        log.info("用户接口访问路由：{}", uri);
        //不需要拦截的路径
        if (ignoreUriBeforeLoginMap.containsKey(uri)) {
            return true;
        }
        //判断是否有权限的路径
        UserDetail userDetail = UserDetailUtils.getUserDetail();
        if (userDetail == null || userDetail.getUserInfo() == null) {
            throw new CommonException(ErrorCode.LOGIN_INVALID.getErrorCode(), "登录信息不存在，请登录");
        }

        //不需要拦截的路径
        if (ignoreUriAfterLoginMap.containsKey(uri)) {
            return true;
        }

        if (userDetail.getUserInfo().getSuperAdmin().intValue() == 1) {
            //超级管理员不限制
            return true;
        }
        Map<String, Object> menuUrlMap = userDetail.getMenuUrlMap();
        if (menuUrlMap == null || menuUrlMap.isEmpty()) {
            throw new CommonException(ErrorCode.SYSTEM_ERROR.getErrorCode(), "sorry，您无该页面的访问权限，请联系超级管理员！");
        }
        if (StringUtils.isNotBlank(isControl)) {
            //说明是操作项
            //如果匹配上了，说明是没权限
            if (menuUrlMap.containsKey(uri)) {
                throw new CommonException(ErrorCode.SYSTEM_ERROR.getErrorCode(), "sorry，您无该页面的访问权限，请联系超级管理员！");
            }
        } else {
            //说明是页面
            //如果没有匹配上，说明没权限
            if (!menuUrlMap.containsKey(uri)) {
                throw new CommonException(ErrorCode.SYSTEM_ERROR.getErrorCode(), "sorry，您无该页面的访问权限，请联系超级管理员！");
            }
        }

        return true;
    }
}
